SystemOrganization addCategory: #'NativeBoost-Tests'!

NBExternalHeapManager subclass: #NBMockExternalHeapManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBMockExternalHeapManager methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 9/25/2010 00:20'!
primAllocatePage: numBytes

	^ 0! !

!NBMockExternalHeapManager methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 9/25/2010 00:21'!
primFreePage: aMemoryPage

	^ 0! !

TestCase subclass: #NBCoreTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'AJx86Registers'
	category: 'NativeBoost-Tests'!

!NBCoreTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/20/2011 04:12'!
testHasTrailerInstalled

	
	"Make sure compiled methods trailer installed"
	
	self assert: (CompiledMethodTrailer trailerKinds includes: #NativeCodeTrailer )! !

!NBCoreTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/20/2011 04:05'!
testPlatformId

	"sometimes i forget to set platform ID, when building plugin"
	
	self assert: NativeBoost platformId ~= 0! !

TestCase subclass: #NBExternalAddressTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBExternalAddressTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/4/2011 08:25'!
testAddressAndStruct

	| struct addr string |
	
	struct := NBTestStructure2 new.
	
	self assert: struct addr class == NBExternalAddress .  
	
	addr := NativeBoost allocate: 10.
	[
		addr writeString: 'abcde'.

		struct addr: addr.
		
		self assert: addr = struct addr.

		string := struct addr readString .		
	] ensure: [ NativeBoost free: addr ].

	self assert: string = 'abcde'
! !

!NBExternalAddressTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/13/2010 22:51'!
testMemoryAccess

	| addr |
	
	addr := NativeBoost allocate: 10.
	[
		1 to: 10 do: [:i |
			addr byteAt: i-1 put: i ].
	
		1 to: 10 do: [:i |
			self assert: (addr byteAt: i-1) = i ].
	] ensure: [ NativeBoost free: addr ]
! !

!NBExternalAddressTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/4/2011 07:29'!
testStringAccess

	| addr string |
	
	addr := NativeBoost allocate: 10.
	[
		addr writeString: 'abcde'.
		string := addr readString. 

	] ensure: [ NativeBoost free: addr ].

	self assert: string = 'abcde'
! !

!NBExternalAddressTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/4/2011 08:50'!
testStructCopy

	| struct addr string struct2 |
	
	struct := NBTestStructure new.
	
	struct byte: 5;
		long: 10;
		short: 20.
		
	addr := NativeBoost allocate: NBTestStructure instanceSize.
	
	NativeBoost memCopy: struct to: addr size:  NBTestStructure instanceSize.
	struct2 := NBTestStructure fromPointer: addr.
	NativeBoost free: addr.
		  

	self assert: struct2 byte = 5.
	self assert: struct2 long = 10.
	self assert: struct2 short = 20.! !

TestCase subclass: #NBExternalHeapManagerTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBExternalHeapManagerTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/5/2011 07:20'!
newHeap
	^  NBMockExternalHeapManager new! !

!NBExternalHeapManagerTests methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 10/5/2010 20:47'!
testAllocate
	" 
		self new testAllocate 
	
	"
	| heap address size |
	
	size := 100.
	heap := self newHeap.
	
	address := heap allocate: size.
	
	self assert:  ( heap isValidAddress: address size: size ) .
	self assert:   heap totalAllocatedMemory = size.
	self assert:   heap numPages = 1 .
	self assert:  heap numAllocatedBlocks = 1.

	heap free: address.

	self assert: heap numAllocatedBlocks = 0.
	self assert: heap numPages = 0 .
	self assert: heap numBlocks = 0 .
	self assert:  (heap isValidAddress: address size: size) not .
	
	self assert: heap totalAllocatedMemory = 0.
	
	
	! !

!NBExternalHeapManagerTests methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 10/5/2010 20:45'!
testAllocate2
	" 
		self new testAllocate2 
	
	"
	| heap addr1 addr2 |
	
	heap := self newHeap.

	addr1 := heap allocate: 1.
	addr2 := heap allocate: 2.
	
	self assert:  heap totalAllocatedMemory = 3 .
	self assert:  heap numAllocatedBlocks = 2 .

	heap free: addr1.

	self assert:  heap numAllocatedBlocks = 1.
	self assert:  heap totalAllocatedMemory = 2 .

	heap free: addr2.
	
	self assert:  heap numPages = 0 .
	self assert: heap numBlocks = 0 .
	self assert:  heap totalAllocatedMemory = 0 .
	
	
	
! !

TestCase subclass: #NBFFICallbackTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'AJx86Registers'
	category: 'NativeBoost-Tests'!

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 9/26/2010 05:38'!
benchQSort

	| bytes memBytes callback callback2 times |
	
	bytes := #[ 2 5 3 10 39 4 80 203 94 215 30 60 80 20 1 4 0 ].
	memBytes := NativeBoost allocate: bytes size.
	
	callback := self createNativeCallback.
	callback2 := NBQSortCallback on: [:a :b | (a byteAt: 0) - (b byteAt: 0) ].

	[ 
		callback install.
		
		times := { 
			[ bytes copy sort ].
	  		[ self primNativeQSort: bytes copy compare: callback address ].
			[ NativeBoost memCopy: bytes to:  memBytes size: bytes size. 
			  self primStQSort: memBytes len: bytes size compare: callback2 ].
		}
		collect: [:block |
			[ 100000 timesRepeat: block ] timeToRun ].

	] ensure:  [ callback uninstall. NativeBoost free: memBytes ].

	^ times

! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/4/2010 01:12'!
checkCallback: arr1 with: arr2 compare: nativeCallback

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	

	^ NBFFICallout cdecl: #(long  (NBExternalAddress nativeCallback, byte* arr1, byte* arr2 ))
		emitCall: [:gen |
			gen asm pop: EAX;
				call: EAX
			] ! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/4/2010 00:45'!
checkCallbackAddress:  nativeCallbackAddr

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	

	^ NBFFICallout cdecl: #(ulong  (NBExternalAddress nativeCallbackAddr))
		emitCall: [:gen | 	gen asm pop: EAX ] 
"			options: #( - optAllowByteArraysPtr +optAllowExternalAddressPtr  ) "! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/14/2010 05:54'!
checkEmitFetchClass
"
	checker for fetching the class code emitting
"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	

	^ NBFFICallout cdecl: #(oop  ())
		emitCall: [:gen |
			gen emitFetchClass: NBExternalAddress
			]! !

!NBFFICallbackTests methodsFor: 'native callbacks' stamp: 'Igor.Stasenko 9/26/2010 05:36'!
createNativeCallback
	"
	The routine must compare the elements, then return one of the following values:
	< 0 elem1 less than elem2 
	0 elem1 equivalent to elem2 
	> 0 elem1 greater than elem2 
	"
	^ NBNativeFunctionGen 
		cdecl: #( int (byte* a, byte * b) )
		emit: [:gen | | asm |
			asm := gen asm.

			asm 
				mov: (gen arg: #b) to: EAX;
				mov: EAX ptr8 to: CL;
				mov: (gen arg: #a) to: EAX;
				mov: EAX ptr8 to: AL;
				sub: AL with: CL;
				movsx: EAX with: AL  "sign-extend the result"
		]
! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/5/2010 16:55'!
dummyMethod: aParam

	^ aParam! !

!NBFFICallbackTests methodsFor: 'qsort callouts' stamp: 'Igor.Stasenko 9/26/2010 05:01'!
primNativeQSort: base compare: nativeCallback
"Here, the base must be a ByteArray instance 
	num, the number of elements in it
	width = 4

Call qsort function:	
	void qsort(
	   void *base, - array ptr
	   size_t num,  - number of elements
	   size_t width,  - size of element
	   int (__cdecl *compare )(const void *, const void *) 
	);

	"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(void qsort (byte* base, NBByteArraySize base, 1, NBExternalAddress nativeCallback))
		module: NativeBoost CLibrary
! !

!NBFFICallbackTests methodsFor: 'qsort callouts' stamp: 'Igor.Stasenko 9/26/2010 05:01'!
primStQSort: base len: size compare: qsortCallback

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(void qsort (NBExternalAddress base, ulong size, 1, NBQSortCallback qsortCallback))
		module: NativeBoost CLibrary
! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/4/2010 00:31'!
readByte: array

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(byte (byte * array))
		emitCall: [:gen |
			gen asm pop: EAX;
				mov: EAX ptr8 to: AL
			] ! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/4/2010 00:36'!
readByte: array at: index

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(byte (byte * array, ulong index))
		emitCall: [:gen |
			gen asm pop: EAX;
				pop: ECX;
				add: EAX with: ECX;
				mov: EAX ptr8 to: AL
			] ! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/5/2011 12:50'!
testCBGeneratedCode
	<expectedFailure>
	
	| code addr trunk |

	code := NBQSortCallback callbackCode code.
	
	addr := NBQSortCallback callbackEnterAddress asUnsignedLong.
	trunk := NBQSortCallback trunkCode.
	
	self assert: (trunk bytes unsignedLongAt: 1+ (trunk offsetAt: #cbEnterAddress) bigEndian: false) = addr.
	
	
	! !

!NBFFICallbackTests methodsFor: 'language-side callback' stamp: 'IgorStasenko 8/5/2011 12:50'!
testCallbackCounter
	<expectedFailure>
	| callback orig bytes result ctr |
	
	"There was a bug in callback code, which crashed VM
	due to override of callgate saved method oop and return address,
	and GC, happened during callback"
	
	self assert: false.
	self error: 'Dont''t run me'.   "HAVE TO REVIEW. CRASHING VM"

 
	orig := #[ 2 5 55 23 67 23 ] copy.
	bytes := NativeBoost allocate: orig size.
	
	NativeBoost memCopy: orig to: bytes  size: orig size.

	ctr := 0.
	callback := NBQSortCallback on: [:a :b |
		NativeBoost insideCallback ifTrue: [ ctr := ctr+ 1].  "this involves using callgate" 
		Smalltalk garbageCollect. "this involves GC"
		 (a byteAt: 0) - (b byteAt: 0) ].
	
	self primStQSort: bytes len: orig size compare: callback.

	result := ByteArray new: orig size.
	NativeBoost memCopy: bytes to: result size: orig size.
	NativeBoost free: bytes.

	self assert: (ctr > 0).		
	self assert: (orig sort reversed = result)



! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/5/2010 17:00'!
testCreateContext

	| context |
	
	context := 
		MethodContext 
			sender: nil
			receiver: 1
			method: (self class compiledMethodAt: #dummyMethod: )
			arguments: #( 55 ).
			
		
	^ context! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/4/2010 00:14'!
testEmitFetchClass

   self assert: (self checkEmitFetchClass ==  NBExternalAddress)! !

!NBFFICallbackTests methodsFor: 'native callbacks' stamp: 'Igor.Stasenko 9/26/2010 05:38'!
testInstallCallback

	| callback |
	
	callback := self createNativeCallback.

	callback install.
	self assert:(callback address notNil).
	callback uninstall.! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/14/2010 01:03'!
testMemCopy

	| mem bytes |


	bytes := #[ 1 2 3 4 5 6 7 8 9 10 ].
	
	mem := NativeBoost allocate: bytes size.
	
	[
	NativeBoost memCopy: bytes to: mem size: bytes size.
	
	1 to: bytes size do: [:i |
		self assert: (bytes at:i) = (mem byteAt: i-1)
		].
	] ensure: [ NativeBoost free: mem ]! !

!NBFFICallbackTests methodsFor: 'native callbacks' stamp: 'Igor.Stasenko 9/26/2010 05:38'!
testNativeCallback

	| orig bytes callback |
	
	orig := #[ 2 5 3 10 39 4 80 ] copy.
	bytes := orig copy.
	
	callback := self createNativeCallback.
	
	[ 
		callback install.
		self primNativeQSort: bytes compare: callback address.
	] ensure:  [ callback uninstall ].

	self assert: (orig sort reversed = bytes)


! !

!NBFFICallbackTests methodsFor: 'language-side callback' stamp: 'IgorStasenko 8/5/2011 12:49'!
testQSortCallbackST
	<expectedFailure>
	
	| callback orig bytes result |
	
	self assert: false. 
	self error: 'Dont''t run me'.   "HAVE TO REVIEW. CRASHING VM"

	orig := #[ 2 5 3 10 39 4 80 ] copy.
	bytes := NativeBoost allocate: orig size.
	
	NativeBoost memCopy: orig to: bytes  size: orig size.

	callback := NBQSortCallback on: [:a :b | (a byteAt: 0) - (b byteAt: 0) ].
	
	self primStQSort: bytes len: orig size compare: callback.

	result := ByteArray new: orig size.
	NativeBoost memCopy: bytes to: result size: orig size.
	NativeBoost free: bytes.
		
	self assert: (orig sort reversed = result)



! !

!NBFFICallbackTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/4/2010 00:33'!
testReadByte

	| arr |
	
	arr := #(240 2 3 4 ) asByteArray.
	
	self assert: (self readByte: arr) == 240! !

!NBFFICallbackTests methodsFor: 'native callbacks' stamp: 'Igor.Stasenko 9/26/2010 05:38'!
testcallbackAddress

	| callback addr1 addr2 |

	callback := self createNativeCallback.
	
	[ 
		callback install.
		addr1 := callback address value.
		addr2 := self checkCallbackAddress: callback address.
	] ensure:  [ callback uninstall ].

	self assert: (addr1 = addr2)! !

TestCase subclass: #NBFFICalloutTests
	instanceVariableNames: ''
	classVariableNames: 'CLASSVAR TYPEVAR'
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBFFICalloutTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/3/2010 23:33'!
arraySize2: array with: param2
	
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(ulong (NBByteArraySize array, long param2))
		emitCall: [:gen |
			"pop the array size"
			gen asm pop: gen asm EAX
		]
! !

!NBFFICalloutTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/3/2010 23:27'!
arraySize: array
	
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(ulong (NBByteArraySize array))
		emitCall: [:gen |
			"pop the array size"
			gen asm pop: gen asm EAX
		]
! !

!NBFFICalloutTests methodsFor: 'tests' stamp: 'IgorStasenko 8/7/2011 04:03'!
nativeFailWithCode: aCode

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #(void (uint aCode) ) emitCall: [:gen :proxy :asm |
			asm pop: asm EAX.
			gen failWithCode: asm EAX.
		]
! !

!NBFFICalloutTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/30/2010 11:14'!
nbBindingOf: aName

	^ self class bindingOf: aName asSymbol! !

!NBFFICalloutTests methodsFor: 'tests' stamp: 'IgorStasenko 8/3/2011 08:24'!
returnParamUsingStackPtr: aParam

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin >
	
	^ NBFFICallout cdecl: #( oop (oop aParam) ) emitCall: [:gen |
		gen asm mov: gen asm ESP ptr to: gen asm EAX ]
	options: #( + optUseStackPointer optNoAlignment - optDirectProxyFnAddress ).
	! !

!NBFFICalloutTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/10/2011 18:27'!
returnReceiver: x arg2: y
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	

	^ NBFFICallout cdecl: #( oop (oop self , oop x, oop y)) emitCall: [
		:gen :proxy :asm |
			proxy receiver
		] "options: #( optDebug )"! !

!NBFFICalloutTests methodsFor: 'tests' stamp: 'Igor.Stasenko 5/3/2010 23:31'!
testArraySize

	| arr |
	
	arr := ByteArray new: 100.

	self assert: (self arraySize: arr) = 100.

	self assert: (self arraySize2: arr with: 20) = 100.! !

!NBFFICalloutTests methodsFor: 'tests' stamp: 'IgorStasenko 8/6/2011 18:36'!
testFailCode

	self assert: 
	(	
		[ self nativeFailWithCode: 1234. false ] on: NBNativeCodeError do: [:ex |
			ex return: ex errorCode = 1234 ]
	)
	! !

!NBFFICalloutTests methodsFor: 'tests' stamp: 'IgorStasenko 8/10/2011 18:27'!
testProxyReceiver

	
	self assert: 
		(self returnReceiver: 10 arg2: 20) == self! !

!NBFFICalloutTests methodsFor: 'tests' stamp: 'Igor.Stasenko 5/1/2010 12:02'!
testUseStackPointer


	self assert: (self returnParamUsingStackPtr: 100) = 100.
! !

TestCase subclass: #NBFnSpecParserTest
	instanceVariableNames: 'types'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBFnSpecParserTest methodsFor: 'as yet unclassified' stamp: 'JavierPimas 2/3/2012 19:37'!
newParser
	^ NBFnSpecParser new! !

!NBFnSpecParserTest methodsFor: 'as yet unclassified' stamp: 'JavierPimas 2/3/2012 19:42'!
testParseAnonFn

	 | tokenizedFnSpec args |
	tokenizedFnSpec := self newParser parseAnonFunction: 
		#(int * * ( 0, nil, -10, FOO_BAR , int a, int* _b, char** c, void* * * d_)).
		
	self assert: tokenizedFnSpec returnType = #('int' '*' '*').
	
	args := tokenizedFnSpec arguments.
	self assert: args size = 8.

	self assert: (args at: 1) = #('0'). 
	self assert: (args at: 2) = #('nil'). 
	self assert: (args at: 3) = #('-10'). 
	self assert: (args at: 4) = #('FOO_BAR'). 
	self assert: (args at: 5) = #('int' 'a'). 
	self assert: (args at: 6) = #('int' '*' '_b'). 
	self assert: (args at: 7) = #('char' '*' '*' 'c'). 
	self assert: (args at: 8) = #('void' '*' '*' '*' 'd_' ). 
! !

!NBFnSpecParserTest methodsFor: 'as yet unclassified' stamp: 'JavierPimas 2/3/2012 19:42'!
testParseNamedFn

	 | tokenizedFnSpec args |
	tokenizedFnSpec := self newParser parseNamedFunction: 
		#(int * * function_name ( 0, nil, -10, FOO_BAR , int a, int* _b, char** c, void* * * d_)).
		
	self assert: tokenizedFnSpec functionName = 'function_name'.
	self assert: tokenizedFnSpec returnType = #('int' '*' '*').
	
	args := tokenizedFnSpec arguments.
	self assert: args size = 8.

	self assert: (args at: 1) = #('0'). 
	self assert: (args at: 2) = #('nil'). 
	self assert: (args at: 3) = #('-10'). 
	self assert: (args at: 4) = #('FOO_BAR'). 
	self assert: (args at: 5) = #('int' 'a'). 
	self assert: (args at: 6) = #('int' '*' '_b'). 
	self assert: (args at: 7) = #('char' '*' '*' 'c'). 
	self assert: (args at: 8) = #('void' '*' '*' '*' 'd_' ). 
! !

!NBFnSpecParserTest methodsFor: 'as yet unclassified' stamp: 'JavierPimas 2/3/2012 19:44'!
testParseNoArgsFn

	 | tokenizedFnSpec args |
	tokenizedFnSpec := self newParser parseNamedFunction:
		#(int * * function_name ( void )).
		
	self assert: tokenizedFnSpec functionName = 'function_name'.
	self assert: tokenizedFnSpec returnType = #('int' '*' '*').
	
	args := tokenizedFnSpec arguments.
	self assert: args size = 0.
! !

TestCase subclass: #NBFnSpecTests
	instanceVariableNames: ''
	classVariableNames: 'CLASSVAR TYPEVAR'
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBFnSpecTests methodsFor: 'tests' stamp: 'JavierPimas 2/4/2012 13:27'!
checkType: type class: typeClass arity: ptrArity stackIndex: stackIndex

	self assert: type class == typeClass.
	self assert: type pointerArity = ptrArity.
	self assert: type loader class == NBSTMethodArgumentLoader.
	self assert: type loader stackIndex = stackIndex ! !

!NBFnSpecTests methodsFor: 'tests' stamp: 'JavierPimas 2/4/2012 13:27'!
checkType: type class: typeClass value: const
	self assert: type class == typeClass.
	self assert: type value = const! !

!NBFnSpecTests methodsFor: 'tests' stamp: 'JavierPimas 2/4/2012 18:04'!
testBuildingFnSpec

	| type tokensSpec bindedSpec s |
	tokensSpec := NBFnSpecParser new
		parseAnonFunction: #( long* (int self ,  String arg1, void**arg2, TYPEVAR arg1, char arg3, 0, nil, true, false , CLASSVAR ) ).
		
	CLASSVAR := 100.
	TYPEVAR := 'long'.

	bindedSpec := tokensSpec asMethodBindableSpec targetClass: self class; methodArguments: #( 'arg1' 'arg2' 'arg3' ); resolve.

	type := bindedSpec returnType.
	self assert: (type class == NBSignedLong).	
	self assert: (type pointerArity = 1).
	
	self assert: (bindedSpec arguments size = 10).
	s := bindedSpec arguments readStream.
"int self"		self checkType: s next class: NBSignedLong arity: 0 stackIndex: 3 .
"String arg1"	self checkType: s next class: NBExternalString arity: 0 stackIndex: 2 .
"void**arg2"	self checkType: s next class: NBVoid arity: 2 stackIndex: 1.
"TYPEVAR arg1"	self checkType: s next class: NBSignedLong arity: 0 stackIndex: 2.
"char arg3"		self checkType: s next class: NBUnsignedChar arity: 0 stackIndex: 0.
"0"				self checkType: s next class: NBFFIConst value: 0.
"nil"				self checkType: s next class: NBFFIConst value: 0.
"true"			self checkType: s next class: NBFFIConst value: 1.
"false"			self checkType: s next class: NBFFIConst value: 0.
"CLASSVAR"	self checkType: s next class: NBFFIConst value: 100.

! !

TestCase subclass: #NBObjectFormat32Tests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'AJx86Registers'
	category: 'NativeBoost-Tests'!

!NBObjectFormat32Tests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 12/21/2011 14:26'!
getFloatPtrOffset: aFloat
	"return the difference between float oop and pointer to its stored floating-point value ,
	must be base header size"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #( ulong (oop aFloat, NBFloatPtr aFloat))
	emitCall: [:gen |
		| asm |
		asm := gen asm.
		asm pop: asm ECX. "oop"
		asm pop: asm EAX. "float ptr"
		asm sub: asm EAX with: asm ECX
	]! !

!NBObjectFormat32Tests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/20/2010 09:19'!
pushDouble: aFloat

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #( double (double aFloat))
	emitCall: [:gen |
		| asm |
		asm := gen asm.
		asm fld: asm ESP ptr64		
	]! !

!NBObjectFormat32Tests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 9/29/2010 11:07'!
pushFloat: aFloat

	<primitive: #primitiveNativeCall module: #NativeBoostPlugin>
	
	^ NBFFICallout cdecl: #( double (float aFloat))
	emitCall: [:gen |
		| asm |
		asm := gen asm.
		asm fld: asm ESP ptr32		
	]! !

!NBObjectFormat32Tests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/23/2011 10:46'!
testCompactFloats

	| idx format |
	format := NBObjectFormat32 new.
	idx := Smalltalk compactClassesArray indexOf: Float ifAbsent: [ ^ nil ].
	idx := idx bitShift: format compactClassIndexShift.

	NativeBoost enableNativeCode.
	Float allInstances do: [:each |
		| oopIdx |
		oopIdx := (NBObjectFormat32 headerOf: each) bitAnd: format compactClassMask.
		self assert: (oopIdx = idx) ]! !

!NBObjectFormat32Tests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 12/21/2011 14:21'!
testPushingFloatPointers

	self assert: (self getFloatPtrOffset: 10.0) =  NBObjectFormat current baseHeaderSize.
! !

!NBObjectFormat32Tests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/20/2010 09:19'!
testPushingFloats

	self assert: (self pushFloat: 100.0) = 100.0.
	self assert: (self pushDouble: 100.0) = 100.0.! !

TestCase subclass: #NativeCodeTests
	instanceVariableNames: 'did'
	classVariableNames: ''
	poolDictionaries: 'AJx86Registers'
	category: 'NativeBoost-Tests'!

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/5/2011 08:22'!
createMoveTestCode

	^ NBFFICallout new
		anonSpec: #( int ());
		cdecl;
		generate: [:gen :proxy :asm |
			| methodOop |
			
			methodOop := gen reserveTemp.
			
			proxy primitiveMethod.
			asm mov: asm EAX to: methodOop.
			
			"call fullGC to trigger memory compaction"
			proxy fullGC.
			
			"return the difference between old primitiveMethod oop value and new one"
			proxy primitiveMethod.
			asm sub: asm EAX with: methodOop.

	].! !

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/2/2010 10:56'!
install: nativeCode into: method

	| trailer  newMethod |
	trailer := CompiledMethodTrailer new.

	trailer nativeCode: nativeCode platformId: NativeBoost platformId sourcePointer: method trailer sourcePointer.
	newMethod := method copyWithTrailerBytes: trailer.
	
	method methodClass methodDict at: method selector put: newMethod.
	
! !

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/11/2010 18:21'!
movableStub
	<primitive: 'primitiveNativeCall' module: 'NativeBoostPlugin'>
	^ 999! !

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/3/2010 21:31'!
testArgumentOffsets

	NBNativeFunctionGen 
		cdecl: #( int (byte a, void * b, double c) )
		emit: [:gen |
			| arg |

			arg := gen arg: #c.
			self assert: (arg isMem).
			self assert: (arg displacement value = 8).

			arg := gen arg: #b.
			self assert: (arg isMem).
			self assert: (arg displacement value = 16).

			arg := gen arg: #a.
			self assert: (arg isMem).
			self assert: (arg displacement value = 20).
			
		]! !

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/5/2011 08:24'!
testMovableStuff
	"test that if native code calls a VM function which triggers
	a full gc and relocates a native code, it will survive the move,
	
	a native code should return a difference between old and new primitive method,
	in case if its moved, the difference will be nonzero"
	
	| code |

	code := self createMoveTestCode.
	" we should not crash here ;) "
	^ (1 to: 10) collect: 
		[:each| 
			Array new: (Random new nextInt: 100). 
			self install: code into: (self class methodDict at: #movableStub).
		  
			self movableStub  
		]
	
	! !

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/30/2010 20:03'!
testProxyFunctions

	| functions |
	
	functions := NBInterpreterProxy functions.
	functions valuesDo: [:fn |
		self assert: (NBInterpreterProxy canUnderstand: fn selector ).
	]! !

!NativeCodeTests methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 4/28/2010 19:52'!
testStructAccess

	| struct |
	
	struct := NBTestStructure new.
	
	struct byte: 10.
	struct short: -20.
	struct long: 100.
	struct float: 1.0.
	struct double: 2.0.
	
	self assert: (struct byte = 10).
	self assert: (struct short = -20).	
	self assert: (struct long = 100).	
	self assert: (struct float = 1.0).	
	self assert: (struct double = 2.0).	
	! !

NBExternalStructure variableByteSubclass: #NBTestStructure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBTestStructure class methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 5/19/2010 09:05'!
fieldsDesc
" self createAccessors "
	^ #(
		byte byte;
		short short;
		long long;
		float float;
		double double;
		)! !

NBExternalStructure variableByteSubclass: #NBTestStructure2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NativeBoost-Tests'!

!NBTestStructure2 class methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/4/2011 08:22'!
fieldsDesc
" self createAccessors "
	^ #(
		NBExternalAddress addr
		)! !
